@@ -478,8 +478,8 @@ gcal_edit_dialog_action_button_clicked (GtkWidget *widget,
     }
   else
     {
-      GDateTime *start_date;
-      GDateTime *end_date;
+      GDateTime *start_date, *end_date;
+      gboolean all_day;
       gchar *note_text;
 
       /* Update summary */
@@ -494,10 +494,24 @@ gcal_edit_dialog_action_button_clicked (GtkWidget *widget,
       g_free (note_text);
 
       /* Update all day */
-      gcal_event_set_all_day (dialog->event,
-                              gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->all_day_check)));
+      all_day = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->all_day_check));
 
-      /* Update start & end dates */
+      gcal_event_set_all_day (dialog->event, all_day);
+
+      /* By definition, all day events are always UTC */
+      if (all_day)
+        {
+          GTimeZone *utc = g_time_zone_new_utc ();
+
+          gcal_event_set_timezone (dialog->event, utc);
+
+          g_clear_pointer (&utc, g_time_zone_unref);
+        }
+
+      /*
+       * Update start & end dates. The dates are already translated to the current
+       * timezone.
+       */
       start_date = gcal_edit_dialog_get_date_start (dialog);
       end_date = gcal_edit_dialog_get_date_end (dialog);
 
@@ -587,6 +601,7 @@ gcal_edit_dialog_set_event (GcalEditDialog *dialog,
       if (!event)
         goto out;
 
+      all_day = gcal_event_get_all_day (event);
       source = gcal_event_get_source (event);
 
       /* Clear event data */
@@ -617,7 +632,10 @@ gcal_edit_dialog_set_event (GcalEditDialog *dialog,
 
       /* retrieve start and end dates */
       date_start = gcal_event_get_date_start (event);
+      date_start = all_day ? g_date_time_ref (date_start) : g_date_time_to_local (date_start);
+
       date_end = gcal_event_get_date_end (event);
+      date_end = all_day ? g_date_time_ref (date_end) : g_date_time_to_local (date_end);
 
       /* date */
       gcal_date_selector_set_date (GCAL_DATE_SELECTOR (dialog->start_date_selector), date_start);
@@ -628,10 +646,8 @@ gcal_edit_dialog_set_event (GcalEditDialog *dialog,
       gcal_time_selector_set_time (GCAL_TIME_SELECTOR (dialog->end_time_selector), date_end);
 
       /* all_day  */
-      all_day = gcal_event_get_all_day (event);
-
-      gtk_widget_set_sensitive (dialog->start_time_selector, !all_day);
-      gtk_widget_set_sensitive (dialog->end_time_selector, !all_day);
+      gtk_widget_set_sensitive (dialog->start_time_selector, dialog->writable && !all_day);
+      gtk_widget_set_sensitive (dialog->end_time_selector, dialog->writable && !all_day);
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->all_day_check), all_day);
 
       /* location */
@@ -644,6 +660,9 @@ gcal_edit_dialog_set_event (GcalEditDialog *dialog,
 
       gcal_edit_dialog_set_writable (dialog, gcal_manager_is_client_writable (dialog->manager, source));
 
+      g_clear_pointer (&date_start, g_date_time_unref);
+      g_clear_pointer (&date_end, g_date_time_unref);
+
 out:
       g_object_notify (G_OBJECT (dialog), "event");
 
@@ -662,17 +681,25 @@ gcal_edit_dialog_set_manager (GcalEditDialog *dialog,
     g_object_notify (G_OBJECT (dialog), "manager");
 }
 
-GDateTime*
-gcal_edit_dialog_get_date_start (GcalEditDialog *dialog)
+static GDateTime*
+return_datetime_for_widgets (GcalEditDialog   *dialog,
+                             GcalDateSelector *date_selector,
+                             GcalTimeSelector *time_selector)
 {
+  GTimeZone *tz;
   GDateTime *date;
   GDateTime *time;
   GDateTime *retval;
+  gboolean all_day;
 
-  date = gcal_date_selector_get_date (GCAL_DATE_SELECTOR (dialog->start_date_selector));
-  time = gcal_time_selector_get_time (GCAL_TIME_SELECTOR (dialog->start_time_selector));
+  /* Use UTC timezone for All Day events, otherwise use the event's timezone */
+  all_day = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->all_day_check));
+  tz = all_day ? g_time_zone_new_utc () : g_time_zone_new_local ();
 
-  retval = g_date_time_new (gcal_event_get_timezone (dialog->event),
+  date = gcal_date_selector_get_date (date_selector);
+  time = gcal_time_selector_get_time (time_selector);
+
+  retval = g_date_time_new (tz,
                             g_date_time_get_year (date),
                             g_date_time_get_month (date),
                             g_date_time_get_day_of_month (date),
@@ -680,26 +707,42 @@ gcal_edit_dialog_get_date_start (GcalEditDialog *dialog)
                             g_date_time_get_minute (time),
                             0);
 
+  /*
+   * If the event is not all day, the timezone may be different from UTC or
+   * local. In any case, since we're editing the event in the current timezone,
+   * we should always correct the timezone to the event's timezone.
+   */
+  if (!all_day)
+    {
+      GDateTime *aux = retval;
+
+      retval = g_date_time_to_timezone (aux, gcal_event_get_timezone (dialog->event));
+
+      g_clear_pointer (&aux, g_date_time_unref);
+    }
+
+  g_clear_pointer (&tz, g_time_zone_unref);
+
   return retval;
 }
 
+GDateTime*
+gcal_edit_dialog_get_date_start (GcalEditDialog *dialog)
+{
+
+  g_return_val_if_fail (GCAL_IS_EDIT_DIALOG (dialog), NULL);
+
+  return return_datetime_for_widgets (dialog,
+                                      GCAL_DATE_SELECTOR (dialog->start_date_selector),
+                                      GCAL_TIME_SELECTOR (dialog->start_time_selector));
+}
+
 GDateTime*
 gcal_edit_dialog_get_date_end (GcalEditDialog *dialog)
 {
-  GDateTime *date;
-  GDateTime *time;
-  GDateTime *retval;
+  g_return_val_if_fail (GCAL_IS_EDIT_DIALOG (dialog), NULL);
 
-  date = gcal_date_selector_get_date (GCAL_DATE_SELECTOR (dialog->end_date_selector));
-  time = gcal_time_selector_get_time (GCAL_TIME_SELECTOR (dialog->end_time_selector));
-
-  retval = g_date_time_new (gcal_event_get_timezone (dialog->event),
-                            g_date_time_get_year (date),
-                            g_date_time_get_month (date),
-                            g_date_time_get_day_of_month (date),
-                            g_date_time_get_hour (time),
-                            g_date_time_get_minute (time),
-                            0);
-
-  return retval;
+  return return_datetime_for_widgets (dialog,
+                                      GCAL_DATE_SELECTOR (dialog->end_date_selector),
+                                      GCAL_TIME_SELECTOR (dialog->end_time_selector));
 }
