@@ -29,10 +29,10 @@ struct _GcalEventWidget
   /* properties */
   GDateTime     *dt_start;
   GDateTime     *dt_end;
-  gboolean       has_reminders;
 
   /* internal data */
-  gboolean       read_only;
+  gboolean       clock_format_24h : 1;
+  gboolean       read_only : 1;
   gchar         *css_class;
 
   GcalEvent     *event;
@@ -47,7 +47,7 @@ enum
   PROP_DATE_END,
   PROP_DATE_START,
   PROP_EVENT,
-  PROP_HAS_REMINDERS
+  NUM_PROPS
 };
 
 enum
@@ -176,8 +176,18 @@ gcal_event_widget_set_event_internal (GcalEventWidget *self,
 static void
 gcal_event_widget_init(GcalEventWidget *self)
 {
+  GSettings *settings;
+  gchar *clock_format;
+
+  settings = g_settings_new ("org.gnome.desktop.interface");
+  clock_format = g_settings_get_string (settings, "clock-format");
+  self->clock_format_24h = g_strcmp0 (clock_format, "24h") == 0;
+
   gtk_widget_set_has_window (GTK_WIDGET (self), FALSE);
   gtk_widget_set_can_focus (GTK_WIDGET (self), TRUE);
+
+  g_clear_object (&settings);
+  g_free (clock_format);
 }
 
 static void
@@ -202,14 +212,6 @@ gcal_event_widget_set_property (GObject      *object,
       gcal_event_widget_set_event_internal (self, g_value_get_object (value));
       break;
 
-    case PROP_HAS_REMINDERS:
-      if (self->has_reminders != g_value_get_boolean (value))
-        {
-          self->has_reminders = g_value_get_boolean (value);
-          g_object_notify (object, "has-reminders");
-        }
-      break;
-
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -237,10 +239,6 @@ gcal_event_widget_get_property (GObject      *object,
       g_value_set_object (value, self->event);
       break;
 
-    case PROP_HAS_REMINDERS:
-      g_value_set_boolean (value, self->has_reminders);
-      break;
-
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -253,6 +251,10 @@ gcal_event_widget_finalize (GObject *object)
 
   self = GCAL_EVENT_WIDGET (object);
 
+  /* disconnect signals */
+  g_signal_handlers_disconnect_by_func (self->event, update_color, self);
+  g_signal_handlers_disconnect_by_func (self->event, gtk_widget_queue_draw, self);
+
   /* releasing properties */
   g_clear_pointer (&self->css_class, g_free);
   g_clear_object (&self->event);
@@ -427,15 +429,18 @@ gcal_event_widget_draw (GtkWidget *widget,
   GtkStateFlags state;
   GtkBorder padding;
 
-  gint width, height, layout_height;
-  gint left_gap, right_gap, icon_size = 0;
+  gint width, height, layout_height, x;
+  gint end_gap, icon_size = 0;
+  gboolean is_ltr;
 
   PangoLayout *layout;
   PangoFontDescription *font_desc;
+  gchar *display_text;
 
   self = GCAL_EVENT_WIDGET (widget);
   context = gtk_widget_get_style_context (widget);
   state = gtk_style_context_get_state (context);
+  is_ltr = gtk_widget_get_direction (widget) != GTK_TEXT_DIR_RTL;
 
   gtk_style_context_get_padding (context, state, &padding);
 
@@ -445,59 +450,83 @@ gcal_event_widget_draw (GtkWidget *widget,
   gtk_render_background (context, cr, 0, 0, width, height);
   gtk_render_frame (context, cr, 0, 0, width, height);
 
+  /*
+   * The text displayed in the widget includes the time when the event
+   * is timed, or not if the event is all day.
+   */
+  if (gcal_event_get_all_day (self->event))
+    {
+      display_text = g_strdup (gcal_event_get_summary (self->event));
+    }
+  else
+    {
+      GDateTime *local_start_time;
+      gchar *start_time;
+      local_start_time = g_date_time_to_local (gcal_event_get_date_start (self->event));
+
+      if (self->clock_format_24h)
+        start_time = g_date_time_format (local_start_time, "%R");
+      else
+        start_time = g_date_time_format (local_start_time, "%I:%M %P");
+
+      if (is_ltr)
+        display_text = g_strdup_printf ("(%s) %s", start_time, gcal_event_get_summary (self->event));
+      else
+        display_text = g_strdup_printf ("%s (%s)", gcal_event_get_summary (self->event), start_time);
+
+      g_clear_pointer (&local_start_time, g_date_time_unref);
+      g_clear_pointer (&start_time, g_free);
+    }
+
   /* FIXME for RTL alignment and icons positions */
   gtk_style_context_get (context, state, "font", &font_desc, NULL);
-  layout = gtk_widget_create_pango_layout (widget, gcal_event_get_summary (self->event));
+  layout = gtk_widget_create_pango_layout (widget, display_text);
   pango_layout_set_font_description (layout, font_desc);
   pango_layout_set_ellipsize (layout, PANGO_ELLIPSIZE_END);
   pango_layout_set_width (layout, (width - (padding.left + padding.right) ) * PANGO_SCALE);
   pango_cairo_update_layout (cr, layout);
 
-  left_gap = 0;
-  if (self->has_reminders)
-    {
-      pango_layout_get_pixel_size (layout, NULL, &layout_height);
-      icon_size = layout_height;
-      left_gap = icon_size + padding.left;
-      pango_layout_set_width (layout, (width - (left_gap + padding.left + padding.right) ) * PANGO_SCALE);
-    }
+  /* Icon size */
+  pango_layout_get_pixel_size (layout, NULL, &layout_height);
+  icon_size = 4 * (layout_height / 4);
+
+  end_gap = 0;
+
+  if (gcal_event_has_alarms (self->event))
+    end_gap += icon_size + (is_ltr ? padding.left : padding.right);
 
-  right_gap = 0;
   if (self->read_only)
-    {
-      if (icon_size == 0)
-        {
-          pango_layout_get_pixel_size (layout, NULL, &layout_height);
-          icon_size = layout_height;
-        }
+    end_gap += icon_size + (is_ltr ? padding.left : padding.right);
 
-      right_gap = icon_size + padding.right;
-      pango_layout_set_width (layout, (width - (left_gap + padding.left + padding.right + right_gap) ) * PANGO_SCALE);
-    }
+  x = is_ltr ? padding.left : padding.left + end_gap;
 
-  gtk_render_layout (context, cr, padding.left + left_gap, padding.top, layout);
+  pango_layout_set_width (layout, (width - (padding.left + padding.right + end_gap)) * PANGO_SCALE);
+  gtk_render_layout (context, cr, x, padding.top, layout);
+
+  x += is_ltr ? (width - end_gap - padding.left - padding.right) : (- padding.right - padding.left - icon_size);
 
   /* render reminder icon */
-  if (self->has_reminders)
+  if (gcal_event_has_alarms (self->event))
     {
       GtkIconTheme *icon_theme;
       GtkIconInfo *icon_info;
       GdkPixbuf *pixbuf;
       gboolean was_symbolic;
-      gint multiplier;
 
-      multiplier = icon_size / 16;
       icon_theme = gtk_icon_theme_get_default ();
       icon_info = gtk_icon_theme_lookup_icon (icon_theme,
                                               "alarm-symbolic",
-                                              16 * multiplier,
+                                              icon_size,
                                               0);
       pixbuf = gtk_icon_info_load_symbolic_for_context (icon_info,
                                                         context,
                                                         &was_symbolic,
                                                         NULL);
 
-      gdk_cairo_set_source_pixbuf (cr, pixbuf, padding.left, padding.top + ((icon_size - (16 * multiplier)) / 2));
+      gdk_cairo_set_source_pixbuf (cr, pixbuf, x + padding.left, padding.top);
+
+      x += is_ltr ? (padding.left + icon_size) : (- padding.right - icon_size);
+
       g_object_unref (pixbuf);
       cairo_paint (cr);
     }
@@ -509,26 +538,25 @@ gcal_event_widget_draw (GtkWidget *widget,
       GtkIconInfo *icon_info;
       GdkPixbuf *pixbuf;
       gboolean was_symbolic;
-      gint multiplier;
 
-      multiplier = icon_size / 16;
       icon_theme = gtk_icon_theme_get_default ();
       icon_info = gtk_icon_theme_lookup_icon (icon_theme,
                                               "changes-prevent-symbolic",
-                                              16 * multiplier,
+                                              icon_size,
                                               0);
       pixbuf = gtk_icon_info_load_symbolic_for_context (icon_info,
                                                         context,
                                                         &was_symbolic,
                                                         NULL);
 
-      gdk_cairo_set_source_pixbuf (cr, pixbuf, width - right_gap, padding.top + ((icon_size - (16 * multiplier)) / 2));
+      gdk_cairo_set_source_pixbuf (cr, pixbuf, x + padding.left, padding.top);
       g_object_unref (pixbuf);
       cairo_paint (cr);
     }
 
   pango_font_description_free (font_desc);
   g_object_unref (layout);
+  g_free (display_text);
 
   return FALSE;
 }
@@ -632,19 +660,6 @@ gcal_event_widget_class_init(GcalEventWidgetClass *klass)
                                                         GCAL_TYPE_EVENT,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
-  /**
-   * GcalEventWidget::has-reminders:
-   *
-   * Whether the event has reminders or not.
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_HAS_REMINDERS,
-                                   g_param_spec_boolean ("has-reminders",
-                                                         "Event has reminders",
-                                                         "Wheter the event has reminders set or not",
-                                                         FALSE,
-                                                         G_PARAM_READWRITE));
-
   signals[ACTIVATE] = g_signal_new ("activate",
                                      GCAL_TYPE_EVENT_WIDGET,
                                      G_SIGNAL_RUN_LAST,
@@ -681,23 +696,6 @@ gcal_event_widget_get_read_only (GcalEventWidget *event)
   return event->read_only;
 }
 
-void
-gcal_event_widget_set_has_reminders (GcalEventWidget *event,
-                                     gboolean         has_reminders)
-{
-  g_return_if_fail (GCAL_IS_EVENT_WIDGET (event));
-
-  g_object_set (event, "has-reminders", has_reminders, NULL);
-}
-
-gboolean
-gcal_event_widget_get_has_reminders (GcalEventWidget *event)
-{
-  g_return_val_if_fail (GCAL_IS_EVENT_WIDGET (event), FALSE);
-
-  return event->has_reminders;
-}
-
 /**
  * gcal_event_widget_get_date_end:
  * @self: a #GcalEventWidget
