@@ -91,7 +91,17 @@ get_timezone_from_ical (ECalComponentDateTime *comp)
     }
   else if (comp->tzid)
     {
-      tz = g_time_zone_new (comp->tzid);
+      icaltimezone *zone;
+      gchar *tzid;
+      gint offset;
+
+      zone = icaltimezone_get_builtin_timezone (comp->tzid);
+      offset = icaltimezone_get_utc_offset (zone, comp->value, NULL);
+      tzid = format_utc_offset (offset);
+
+      tz = g_time_zone_new (tzid);
+
+      g_free (tzid);
     }
   else
     {
@@ -102,30 +112,25 @@ get_timezone_from_ical (ECalComponentDateTime *comp)
 }
 
 static ECalComponentDateTime*
-build_component_from_datetime (GcalEvent *self,
-                               GDateTime *dt)
+build_component_from_datetime (GcalEvent    *self,
+                               icaltimezone *current_tz,
+                               GDateTime    *dt)
 {
   ECalComponentDateTime *comp_dt;
-  GDateTime *utf_dt;
 
   if (!dt)
     return NULL;
 
-  utf_dt = g_date_time_to_utc (dt);
-
   comp_dt = g_new0 (ECalComponentDateTime, 1);
   comp_dt->value = NULL;
   comp_dt->tzid = NULL;
 
-  comp_dt->value = datetime_to_icaltime (utf_dt);
-  comp_dt->value->zone = NULL;
+  comp_dt->value = datetime_to_icaltime (dt);
+  comp_dt->value->zone = self->all_day ? icaltimezone_get_utc_timezone () : current_tz;
   comp_dt->value->is_date = self->all_day;
 
-  /* All day events have no timezone */
-  if (!self->all_day)
-    comp_dt->tzid = format_utc_offset (g_date_time_get_utc_offset (dt));
-
-  g_clear_pointer (&utf_dt, g_date_time_unref);
+  /* All day events have UTC timezone */
+  comp_dt->tzid = g_strdup (self->all_day ? "UTC" : icaltimezone_get_tzid (current_tz));
 
   return comp_dt;
 }
@@ -156,6 +161,9 @@ gcal_event_update_uid_internal (GcalEvent *self)
   source_id = self->source ? e_source_get_uid (self->source) : "";
   id = e_cal_component_get_id (self->component);
 
+  /* Clear the previous uid */
+  g_clear_pointer (&self->uid, g_free);
+
   if (id->rid != NULL)
     {
       self->uid = g_strdup_printf ("%s:%s:%s",
@@ -181,42 +189,44 @@ gcal_event_set_component_internal (GcalEvent     *self,
     {
       ECalComponentDateTime start;
       ECalComponentDateTime end;
-      icaltimetype normalized_date;
+      icaltimetype date;
       GDateTime *date_start;
       GTimeZone *zone_start;
       GDateTime *date_end;
       GTimeZone *zone_end;
-      GDateTime *aux;
       gboolean start_is_all_day, end_is_all_day;
       gchar *description;
 
       /* Setup start date */
       e_cal_component_get_dtstart (component, &start);
-      normalized_date = icaltime_normalize (*start.value);
+      date = icaltime_normalize (*start.value);
       zone_start = get_timezone_from_ical (&start);
-      aux = icaltime_to_datetime (&normalized_date);
-      date_start = g_date_time_to_timezone (aux, zone_start);
-      start_is_all_day = datetime_is_date (aux);
+      date_start = g_date_time_new (zone_start,
+                                    date.year, date.month, date.day,
+                                    date.is_date ? 0 : date.hour,
+                                    date.is_date ? 0 : date.minute,
+                                    date.is_date ? 0 : date.second);
+      start_is_all_day = datetime_is_date (date_start);
 
-      self->dt_start = g_date_time_ref (date_start);
+      self->dt_start = date_start;
 
-      g_clear_pointer (&aux, g_date_time_unref);
 
       /* The timezone of the event is the timezone of the start date */
       self->timezone = g_time_zone_ref (zone_start);
 
       /* Setup end date */
       e_cal_component_get_dtend (component, &end);
-      normalized_date = icaltime_normalize (*end.value);
+      date = icaltime_normalize (*end.value);
       zone_end = get_timezone_from_ical (&end);
-      aux = icaltime_to_datetime (&normalized_date);
-      date_end = g_date_time_to_timezone (aux, zone_end);
-      end_is_all_day = datetime_is_date (aux);
+      date_end = g_date_time_new (zone_end,
+                                  date.year, date.month, date.day,
+                                  date.is_date ? 0 : date.hour,
+                                  date.is_date ? 0 : date.minute,
+                                  date.is_date ? 0 : date.second);
+      end_is_all_day = datetime_is_date (date_end);
 
       self->dt_end = g_date_time_ref (date_end);
 
-      g_clear_pointer (&aux, g_date_time_unref);
-
       /* Setup all day */
       self->all_day = start_is_all_day && end_is_all_day;
 
@@ -231,9 +241,7 @@ gcal_event_set_component_internal (GcalEvent     *self,
       g_object_notify (G_OBJECT (self), "location");
       g_object_notify (G_OBJECT (self), "summary");
 
-      g_clear_pointer (&date_start, g_date_time_unref);
       g_clear_pointer (&zone_start, g_time_zone_unref);
-      g_clear_pointer (&date_end, g_date_time_unref);
       g_clear_pointer (&zone_end, g_time_zone_unref);
       g_clear_pointer (&description, g_free);
     }
@@ -678,13 +686,18 @@ gcal_event_set_date_end (GcalEvent *self,
       (!self->dt_end || !dt ||
        (self->dt_end && dt && !g_date_time_equal (self->dt_end, dt))))
     {
-      ECalComponentDateTime *component_dt;
+      ECalComponentDateTime *component_dt, current;
 
       g_clear_pointer (&self->dt_end, g_date_time_unref);
       self->dt_end = g_date_time_ref (dt);
 
+      /* Retrieve the current timezone */
+      e_cal_component_get_dtstart (self->component, &current);
+
       /* Setup the ECalComponent's datetime value */
-      component_dt = build_component_from_datetime (self, dt);
+      component_dt = build_component_from_datetime (self,
+                                                    icaltimezone_get_builtin_timezone_from_tzid (current.tzid),
+                                                    dt);
 
       e_cal_component_set_dtend (self->component, component_dt);
       e_cal_component_commit_sequence (self->component);
@@ -728,13 +741,18 @@ gcal_event_set_date_start (GcalEvent *self,
       (!self->dt_start || !dt ||
        (self->dt_start && dt && !g_date_time_equal (self->dt_start, dt))))
     {
-      ECalComponentDateTime *component_dt;
+      ECalComponentDateTime *component_dt, current;
 
       g_clear_pointer (&self->dt_start, g_date_time_unref);
       self->dt_start = g_date_time_ref (dt);
 
+      /* Retrieve the current timezone */
+      e_cal_component_get_dtstart (self->component, &current);
+
       /* Setup the ECalComponent's datetime value */
-      component_dt = build_component_from_datetime (self, dt);
+      component_dt = build_component_from_datetime (self,
+                                                    icaltimezone_get_builtin_timezone_from_tzid (current.tzid),
+                                                    dt);
 
       e_cal_component_set_dtstart (self->component, component_dt);
       e_cal_component_commit_sequence (self->component);
@@ -794,6 +812,22 @@ gcal_event_set_description (GcalEvent   *self,
     }
 }
 
+/**
+ * gcal_event_has_alarms:
+ * @self: a #GcalEvent
+ *
+ * Retrieves whether the event has alarms or not.
+ *
+ * Returns: %TRUE if @event has alarms, %FALSE otherwise.
+ */
+gboolean
+gcal_event_has_alarms (GcalEvent *self)
+{
+  g_return_val_if_fail (GCAL_IS_EVENT (self), FALSE);
+
+  return e_cal_component_has_alarms (self->component);
+}
+
 /**
  * gcal_event_get_location:
  * @self: a #GcalEvent
@@ -990,28 +1024,31 @@ gcal_event_set_timezone (GcalEvent *self,
 
   if (self->timezone != timezone)
     {
-      GDateTime *new_dtstart;
-
       g_clear_pointer (&self->timezone, g_time_zone_unref);
       self->timezone = g_time_zone_ref (timezone);
 
       /* Swap the timezone from the component start & end dates*/
-      new_dtstart = g_date_time_to_timezone (self->dt_start, timezone);
-      gcal_event_set_date_start (self, new_dtstart);
-
-      if (self->dt_end)
+      if (!self->all_day)
         {
-          GDateTime *new_dtend;
+          GDateTime *new_dtstart;
 
-          new_dtend = g_date_time_to_timezone (self->dt_end, timezone);
-          gcal_event_set_date_end (self, new_dtend);
+          new_dtstart = g_date_time_to_timezone (self->dt_start, timezone);
+          gcal_event_set_date_start (self, new_dtstart);
+
+          if (self->dt_end)
+            {
+              GDateTime *new_dtend;
+
+              new_dtend = g_date_time_to_timezone (self->dt_end, timezone);
+              gcal_event_set_date_end (self, new_dtend);
+
+              g_clear_pointer (&new_dtstart, g_date_time_unref);
+            }
 
           g_clear_pointer (&new_dtstart, g_date_time_unref);
         }
 
       g_object_notify (G_OBJECT (self), "timezone");
-
-      g_clear_pointer (&new_dtstart, g_date_time_unref);
     }
 }
 
