@@ -161,9 +161,6 @@ static gboolean       gcal_month_view_button_release        (GtkWidget      *wid
 static void           gcal_month_view_direction_changed     (GtkWidget        *widget,
                                                              GtkTextDirection  previous_direction);
 
-static gboolean       gcal_month_view_is_child_multiday     (GcalSubscriberView  *subscriber,
-                                                             GcalEventWidget     *child);
-
 static guint          gcal_month_view_get_child_cell        (GcalSubscriberView *subscriber,
                                                              GcalEventWidget    *child);
 
@@ -267,21 +264,28 @@ show_popover_for_position (GcalMonthView *view,
     {
       GDateTime *start_dt, *end_dt;
 
-      start_dt = g_date_time_new_local (priv->date->year, priv->date->month, start_day, 0, 0, 0);
-      end_dt = g_date_time_new_local (priv->date->year, priv->date->month, end_day + 1, 0, 0, 0);
+      start_dt = NULL;
+      end_dt = NULL;
 
       /* Swap dates if start > end */
       if (start_day > end_day)
         {
-          GDateTime *aux = start_dt;
-          start_dt = end_dt;
-          end_dt = aux;
+          gint aux;
+          aux = end_day;
+          end_day = start_day;
+          start_day = aux;
         }
 
+      start_dt = g_date_time_new_utc (priv->date->year, priv->date->month, start_day, 0, 0, 0);
+
+      /* Only setup an end date when days are different */
+      if (start_day != end_day)
+        end_dt = g_date_time_new_utc (priv->date->year, priv->date->month, end_day + 1, 0, 0, 0);
+
       g_signal_emit_by_name (GCAL_VIEW (widget), "create-event", start_dt, end_dt, x, y);
 
-      g_date_time_unref (start_dt);
-      g_date_time_unref (end_dt);
+      g_clear_pointer (&start_dt, g_date_time_unref);
+      g_clear_pointer (&end_dt, g_date_time_unref);
     }
 
   gtk_widget_queue_draw (widget);
@@ -610,10 +614,12 @@ update_list_box_headers (GtkListBoxRow *row,
   row_child = gtk_bin_get_child (GTK_BIN (row));
   row_event = gcal_event_widget_get_event (GCAL_EVENT_WIDGET (row_child));
   row_date = gcal_event_widget_get_date_start (GCAL_EVENT_WIDGET (row_child));
+  row_date = g_date_time_to_local (row_date);
   if (before != NULL)
     {
       before_child = gtk_bin_get_child (GTK_BIN (before));
       before_date = gcal_event_widget_get_date_start (GCAL_EVENT_WIDGET (before_child));
+      before_date = g_date_time_to_local (before_date);
     }
 
   if (!gcal_event_is_multiday (row_event) &&
@@ -643,6 +649,9 @@ update_list_box_headers (GtkListBoxRow *row,
       gtk_list_box_row_set_header (row, vbox);
       g_free (time);
     }
+
+  g_clear_pointer (&before_date, g_date_time_unref);
+  g_clear_pointer (&row_date, g_date_time_unref);
 }
 
 static gboolean
@@ -748,7 +757,7 @@ add_new_event_button_cb (GtkWidget *button,
   gtk_widget_hide (priv->overflow_popover);
 
   day = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (priv->overflow_popover), "selected-day"));
-  start_date = g_date_time_new_local (priv->date->year, priv->date->month, day, 0, 0, 0);
+  start_date = g_date_time_new_utc (priv->date->year, priv->date->month, day, 0, 0, 0);
 
   g_signal_emit_by_name (GCAL_VIEW (user_data), "create-event-detailed", start_date, NULL);
 
@@ -781,7 +790,6 @@ gcal_month_view_class_init (GcalMonthViewClass *klass)
   widget_class->key_press_event = gcal_month_view_key_press;
 
   subscriber_view_class = GCAL_SUBSCRIBER_VIEW_CLASS (klass);
-  subscriber_view_class->is_child_multicell = gcal_month_view_is_child_multiday;
   subscriber_view_class->get_child_cell = gcal_month_view_get_child_cell;
   subscriber_view_class->clear_state = gcal_month_view_clear_state;
 
@@ -1063,7 +1071,7 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
   start_grid_y = get_start_grid_y (widget);
   cell_width = allocation->width / 7.0;
   cell_height = (allocation->height - start_grid_y) / 6.0;
-  vertical_cell_space = cell_height - (padding_bottom + font_height);
+  vertical_cell_space = cell_height - (padding_bottom * 2 + font_height);
 
   for (i = 0; i < 42; i++)
     size_left[i] = vertical_cell_space;
@@ -1080,7 +1088,7 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
   for (l = ppriv->multi_cell_children; l != NULL; l = g_list_next (l))
     {
       gint first_cell, last_cell, first_row, last_row, start, end;
-      gboolean visible;
+      gboolean visible, all_day;
 
       GDateTime *date;
       GcalEvent *event;
@@ -1089,6 +1097,7 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
       child_widget = (GtkWidget*) l->data;
       event = gcal_event_widget_get_event (l->data);
       uuid = gcal_event_get_uid (event);
+      all_day = gcal_event_get_all_day (event);
 
       if (!gtk_widget_is_visible (child_widget) && !g_hash_table_contains (ppriv->hidden_as_overflow, uuid))
         continue;
@@ -1103,12 +1112,15 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
        */
       j = 1;
       date = gcal_event_get_date_start (event);
+      date = all_day ? g_date_time_ref (date) : g_date_time_to_local (date);
 
       if (g_date_time_get_month (date) == priv->date->month)
         j = g_date_time_get_day_of_month (date);
 
       j += priv->days_delay;
 
+      g_clear_pointer (&date, g_date_time_unref);
+
       /*
        * Calculate the first cell position according to the locale
        * and the start date.
@@ -1121,17 +1133,20 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
        */
       j = icaltime_days_in_month (priv->date->month, priv->date->year);
       date = gcal_event_get_date_end (event);
+      date = all_day ? g_date_time_ref (date) : g_date_time_to_local (date);
 
       if (g_date_time_get_month (date) == priv->date->month)
         {
           j = g_date_time_get_day_of_month (date);
 
           /* If the event is all day, we have to subtract 1 to find the the real date */
-          if (gcal_event_get_all_day (event))
+          if (all_day)
             j--;
         }
       j += priv->days_delay;
 
+      g_clear_pointer (&date, g_date_time_unref);
+
       last_cell = 7 * ((j - 1) / 7)+ 6 * priv->k + sw * ((j - 1) % 7);
 
       /*
@@ -1264,9 +1279,11 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
       l = (GList*) value;
       for (aux = l; aux != NULL; aux = g_list_next (aux))
         {
-          child_widget = (GtkWidget*) aux->data;
+          GcalEvent *event;
 
-          uuid = gcal_event_get_uid (gcal_event_widget_get_event (GCAL_EVENT_WIDGET (child_widget)));
+          child_widget = aux->data;
+          event = gcal_event_widget_get_event (aux->data);
+          uuid = gcal_event_get_uid (event);
 
           if (!gtk_widget_is_visible (child_widget) && !g_hash_table_contains (ppriv->hidden_as_overflow, uuid))
             continue;
@@ -1839,22 +1856,31 @@ gcal_month_view_direction_changed (GtkWidget        *widget,
     priv->k = 1;
 }
 
-static gboolean
-gcal_month_view_is_child_multiday (GcalSubscriberView *subscriber,
-                                     GcalEventWidget    *child)
-{
-  return gcal_event_is_multiday (gcal_event_widget_get_event (child));
-}
-
 static guint
 gcal_month_view_get_child_cell (GcalSubscriberView *subscriber,
                                 GcalEventWidget    *child)
 {
+  GcalEvent *event;
   GDateTime *dt;
+  gint cell;
 
-  dt = gcal_event_widget_get_date_start (child);
+  event = gcal_event_widget_get_event (child);
+  dt = gcal_event_get_date_start (event);
 
-  return g_date_time_get_day_of_month (dt);
+  /* Don't adjust the date when the event is all day */
+  if (gcal_event_get_all_day (event))
+    {
+      cell = g_date_time_get_day_of_month (dt);
+    }
+  else
+    {
+      dt = g_date_time_to_local (dt);
+      cell = g_date_time_get_day_of_month (dt);
+
+      g_clear_pointer (&dt, g_date_time_unref);
+    }
+
+  return cell;
 }
 
 static void
