@@ -20,20 +20,21 @@
 
 #include "gcal-search-view.h"
 
-#include "gcal-event-widget.h"
+#include "gcal-event.h"
 #include "gcal-utils.h"
 
 #include <glib/gi18n.h>
 
 typedef struct
 {
-  GcalEventData  *event_data;
-  GtkWidget      *row;
-  gchar          *uuid;
+  GcalEvent *event;
+  GtkWidget *row;
 } RowEventData;
 
-typedef struct
+struct _GcalSearchView
 {
+  GtkPopover      parent;
+
   GtkWidget      *listbox;
   GtkWidget      *stack;
 
@@ -59,7 +60,7 @@ typedef struct
   /* flags */
   gboolean        format_24h;
   gboolean        subscribed;
-} GcalSearchViewPrivate;
+};
 
 enum
 {
@@ -90,17 +91,13 @@ static gint           sort_by_event                             (GtkListBoxRow
                                                                  GtkListBoxRow        *row2,
                                                                  gpointer              user_data);
 
-static gint           compare_events                            (GcalEventData        *ev1,
-                                                                 GcalEventData        *ev2,
-                                                                 time_t               *utc);
-
 static gboolean       show_no_results_page                      (GcalSearchView       *view);
 
 /* private */
 static void           free_row_data                             (RowEventData          *data);
 
-static GtkWidget*     make_row_for_event_data                   (GcalSearchView       *view,
-                                                                 GcalEventData        *data);
+static GtkWidget*     make_row_for_event                        (GcalSearchView       *view,
+                                                                 GcalEvent            *event);
 
 static void           update_view                               (GcalSearchView       *view);
 
@@ -141,7 +138,6 @@ static void           gcal_search_view_thaw                     (ECalDataModelSu
 G_DEFINE_TYPE_WITH_CODE (GcalSearchView,
                          gcal_search_view,
                          GTK_TYPE_POPOVER,
-                         G_ADD_PRIVATE (GcalSearchView)
                          G_IMPLEMENT_INTERFACE (E_TYPE_CAL_DATA_MODEL_SUBSCRIBER,
                                                 gcal_data_model_subscriber_interface_init));
 
@@ -188,15 +184,14 @@ open_event (GtkListBox    *list,
             gpointer       user_data)
 {
   RowEventData *data;
-  ECalComponentDateTime dt;
+  icaltimetype *date_start;
 
   data = g_object_get_data (G_OBJECT (row), "event-data");
+  date_start = datetime_to_icaltime (gcal_event_get_date_start (data->event));
 
-  e_cal_component_get_dtstart (data->event_data->event_component, &dt);
+  g_signal_emit_by_name (user_data, "event-activated", date_start);
 
-  g_signal_emit_by_name (user_data, "event-activated", dt.value);
-
-  e_cal_component_free_datetime (&dt);
+  g_free (date_start);
 }
 
 /**
@@ -220,46 +215,23 @@ sort_by_event (GtkListBoxRow *row1,
                GtkListBoxRow *row2,
                gpointer       user_data)
 {
-  GcalSearchViewPrivate *priv;
+  GcalSearchView *view;
   RowEventData *rd1, *rd2;
-  GcalEventData *ev1, *ev2;
+  GcalEvent *ev1, *ev2;
 
-  priv = gcal_search_view_get_instance_private (GCAL_SEARCH_VIEW (user_data));
+  view = GCAL_SEARCH_VIEW (user_data);
 
   /* retrieve event data */
   rd1 = g_object_get_data (G_OBJECT (row1), "event-data");
   rd2 = g_object_get_data (G_OBJECT (row2), "event-data");
 
-  ev1 = rd1->event_data;
-  ev2 = rd2->event_data;
+  ev1 = rd1->event;
+  ev2 = rd2->event;
 
   if (ev1 == NULL || ev2 == NULL)
       return 0;
 
-  return compare_events (ev1, ev2, &(priv->current_utc_date));
-}
-
-static gint
-compare_events (GcalEventData *ev1,
-                GcalEventData *ev2,
-                time_t        *utc)
-{
-  ECalComponentDateTime date1, date2;
-  gint result;
-
-  e_cal_component_get_dtstart (ev1->event_component, &date1);
-  e_cal_component_get_dtstart (ev2->event_component, &date2);
-
-  if (date1.tzid != NULL)
-    date1.value->zone = icaltimezone_get_builtin_timezone_from_tzid (date1.tzid);
-  if (date2.tzid != NULL)
-    date2.value->zone = icaltimezone_get_builtin_timezone_from_tzid (date2.tzid);
-  result = icaltime_compare_with_current (date1.value, date2.value, utc);
-
-  e_cal_component_free_datetime (&date1);
-  e_cal_component_free_datetime (&date2);
-
-  return result;
+  return gcal_event_compare_with_current (ev1, ev2, &(view->current_utc_date));
 }
 
 /**
@@ -273,15 +245,12 @@ compare_events (GcalEventData *ev1,
 static gboolean
 show_no_results_page (GcalSearchView *view)
 {
-  GcalSearchViewPrivate *priv;
+  view->no_results_timeout_id = 0;
 
-  priv = gcal_search_view_get_instance_private (view);
-  priv->no_results_timeout_id = 0;
-
-  if (priv->query)
-    gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), priv->num_results != 0 ? "results" : "no_results");
+  if (view->query)
+    gtk_stack_set_visible_child_name (GTK_STACK (view->stack), view->num_results != 0 ? "results" : "no_results");
   else
-    gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), "results");
+    gtk_stack_set_visible_child_name (GTK_STACK (view->stack), "results");
 
   return G_SOURCE_REMOVE;
 }
@@ -303,10 +272,7 @@ free_row_data (RowEventData *data)
   if (data->row != NULL)
     gtk_widget_destroy (GTK_WIDGET (data->row));
 
-  g_object_unref (data->event_data->event_component);
-
-  g_free (data->event_data);
-  g_free (data->uuid);
+  g_object_unref (data->event);
   g_free (data);
 }
 
@@ -321,17 +287,11 @@ free_row_data (RowEventData *data)
  * Returns: the newly created #GtkListBoxRow with the event data
  */
 static GtkWidget*
-make_row_for_event_data (GcalSearchView  *view,
-                         GcalEventData   *data)
+make_row_for_event (GcalSearchView *view,
+                    GcalEvent      *event)
 {
-  GcalSearchViewPrivate *priv;
-
-  g_autoptr(GTimeZone) tz;
-  g_autoptr (GDateTime) datetime;
-  g_autoptr (GDateTime) local_datetime;
-  ECalComponentDateTime comp_dt;
-  ECalComponentText summary;
-  GdkRGBA color;
+  GDateTime *local_datetime;
+  GDateTime *date_start;
   GdkPixbuf *pixbuf;
 
   GtkWidget *row;
@@ -345,19 +305,12 @@ make_row_for_event_data (GcalSearchView  *view,
   GtkWidget *name_label;
   GtkWidget *image;
 
-  priv = gcal_search_view_get_instance_private (view);
-
   /* get event color */
-  get_color_name_from_source (data->source, &color);
-  pixbuf = gcal_get_pixbuf_from_color (&color, 16);
+  pixbuf = gcal_get_pixbuf_from_color (gcal_event_get_color (event), 16);
 
   /* make an image of the color */
   image = gtk_image_new_from_pixbuf (pixbuf);
 
-  /* date */
-  e_cal_component_get_dtstart (data->event_component, &comp_dt);
-  e_cal_component_get_summary (data->event_component, &summary);
-
   /* grid & box*/
   row = gtk_list_box_row_new ();
   grid = gtk_grid_new ();
@@ -379,26 +332,19 @@ make_row_for_event_data (GcalSearchView  *view,
   gtk_widget_set_hexpand (name_box, TRUE);
 
   /* start date & time */
-  if (comp_dt.tzid != NULL)
-    tz = g_time_zone_new (comp_dt.tzid);
-  else if (comp_dt.value->zone != NULL)
-    tz = g_time_zone_new (icaltimezone_get_tzid ((icaltimezone*) comp_dt.value->zone));
-  else
-    tz = g_time_zone_new_local ();
-
-  datetime = g_date_time_new (tz,
-                              comp_dt.value->year, comp_dt.value->month, comp_dt.value->day,
-                              comp_dt.value->hour, comp_dt.value->minute, comp_dt.value->second);
-  local_datetime = g_date_time_to_local (datetime);
+  date_start = gcal_event_get_date_start (event);
+  local_datetime = g_date_time_to_local (date_start);
   text = g_date_time_format (local_datetime, "%x");
+
+  /* Date label */
   date_label = gtk_label_new (text);
   gtk_label_set_width_chars (GTK_LABEL (date_label), 11);
   g_free (text);
 
   /* show 'all day' instead of 00:00 */
-  if (comp_dt.value->is_date == 0)
+  if (gcal_event_get_all_day (event))
     {
-      text = g_date_time_format (local_datetime, priv->format_24h ? "%R" : "%r");
+      text = g_date_time_format (local_datetime, view->format_24h ? "%R" : "%r");
       time_label = gtk_label_new (text);
       g_free (text);
     }
@@ -411,13 +357,13 @@ make_row_for_event_data (GcalSearchView  *view,
   gtk_style_context_add_class (gtk_widget_get_style_context (time_label), "dim-label");
 
   /* name label */
-  name_label = gtk_label_new (summary.value);
+  name_label = gtk_label_new (gcal_event_get_summary (event));
   gtk_widget_set_hexpand (name_label, TRUE);
   gtk_widget_set_halign (name_label, GTK_ALIGN_START);
   gtk_label_set_ellipsize (GTK_LABEL (name_label), PANGO_ELLIPSIZE_END);
 
   /* alarm icon */
-  if (e_cal_component_has_alarms (data->event_component))
+  if (e_cal_component_has_alarms (gcal_event_get_component (event)))
     {
       GtkWidget *alarm_icon;
 
@@ -426,7 +372,7 @@ make_row_for_event_data (GcalSearchView  *view,
     }
 
   /* lock icon */
-  if (!gcal_manager_is_client_writable (priv->manager, data->source))
+  if (!gcal_manager_is_client_writable (view->manager, gcal_event_get_source (event)))
     {
       GtkWidget *lock_icon;
 
@@ -445,8 +391,9 @@ make_row_for_event_data (GcalSearchView  *view,
 
   gtk_widget_show_all (row);
 
-  e_cal_component_free_datetime (&comp_dt);
+  g_clear_pointer (&local_datetime, g_date_time_unref);
   g_object_unref (pixbuf);
+
   return row;
 }
 
@@ -462,13 +409,10 @@ make_row_for_event_data (GcalSearchView  *view,
 static void
 update_view (GcalSearchView *view)
 {
-  GcalSearchViewPrivate *priv;
-  priv = gcal_search_view_get_instance_private (view);
+  if (view->no_results_timeout_id != 0)
+    g_source_remove (view->no_results_timeout_id);
 
-  if (priv->no_results_timeout_id != 0)
-    g_source_remove (priv->no_results_timeout_id);
-
-  priv->no_results_timeout_id = g_timeout_add (NO_RESULT_TIMEOUT, (GSourceFunc) show_no_results_page, view);
+  view->no_results_timeout_id = g_timeout_add (NO_RESULT_TIMEOUT, (GSourceFunc) show_no_results_page, view);
 }
 
 static void
@@ -492,7 +436,7 @@ gcal_search_view_class_init (GcalSearchViewClass *klass)
    *
    */
   signals[EVENT_ACTIVATED] = g_signal_new ("event-activated", GCAL_TYPE_SEARCH_VIEW, G_SIGNAL_RUN_LAST,
-                                           G_STRUCT_OFFSET (GcalSearchViewClass, event_activated),
+                                           0,
                                            NULL, NULL, NULL,
                                            G_TYPE_NONE, 1, ICAL_TIME_TYPE);
 
@@ -513,8 +457,8 @@ gcal_search_view_class_init (GcalSearchViewClass *klass)
   /* bind things for/from the template class */
   gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass), "/org/gnome/calendar/search-view.ui");
 
-  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GcalSearchView, stack);
-  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GcalSearchView, listbox);
+  gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GcalSearchView, stack);
+  gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GcalSearchView, listbox);
 
   gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (klass), open_event);
 }
@@ -538,17 +482,17 @@ gcal_data_model_subscriber_interface_init (ECalDataModelSubscriberInterface *ifa
 static void
 gcal_search_view_constructed (GObject *object)
 {
-  GcalSearchViewPrivate *priv;
+  GcalSearchView *view;
   GtkWidget *placeholder_label;
 
-  priv = gcal_search_view_get_instance_private (GCAL_SEARCH_VIEW (object));
+  view = GCAL_SEARCH_VIEW (object);
 
   /* make the listbox sorted */
-  gtk_list_box_set_sort_func (GTK_LIST_BOX (priv->listbox), (GtkListBoxSortFunc) sort_by_event, object, NULL);
-  gtk_list_box_set_header_func (GTK_LIST_BOX (priv->listbox), display_header_func, NULL, NULL);
+  gtk_list_box_set_sort_func (GTK_LIST_BOX (view->listbox), (GtkListBoxSortFunc) sort_by_event, object, NULL);
+  gtk_list_box_set_header_func (GTK_LIST_BOX (view->listbox), display_header_func, NULL, NULL);
 
-  // (gchar)uuid -> (RowEventData)event Hash table
-  priv->uuid_to_event = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, (GDestroyNotify) free_row_data);
+  // gchar* -> RowEventData*
+  view->uuid_to_event = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) free_row_data);
 
   /* don't fill the list with all events on startup */
   gcal_search_view_search (GCAL_SEARCH_VIEW (object), NULL, NULL);
@@ -563,7 +507,7 @@ gcal_search_view_constructed (GObject *object)
                                     NULL);
   gtk_style_context_add_class (gtk_widget_get_style_context (placeholder_label), "dim-label");
 
-  gtk_list_box_set_placeholder (GTK_LIST_BOX (priv->listbox), placeholder_label);
+  gtk_list_box_set_placeholder (GTK_LIST_BOX (view->listbox), placeholder_label);
 }
 
 static void
@@ -572,18 +516,14 @@ gcal_search_view_set_property (GObject       *object,
                                const GValue  *value,
                                GParamSpec    *pspec)
 {
-  GcalSearchViewPrivate *priv;
-
-  priv = gcal_search_view_get_instance_private (GCAL_SEARCH_VIEW (object));
+  GcalSearchView *self = GCAL_SEARCH_VIEW (object);
 
   switch (property_id)
     {
     case PROP_DATE:
       {
-        if (priv->date != NULL)
-          g_free (priv->date);
-
-        priv->date = g_value_dup_boxed (value);
+        g_clear_pointer (&self->date, g_free);
+        self->date = g_value_dup_boxed (value);
         break;
       }
     default:
@@ -598,14 +538,12 @@ gcal_search_view_get_property (GObject       *object,
                                GValue        *value,
                                GParamSpec    *pspec)
 {
-  GcalSearchViewPrivate *priv;
-
-  priv = gcal_search_view_get_instance_private (GCAL_SEARCH_VIEW (object));
+  GcalSearchView *self = GCAL_SEARCH_VIEW (object);
 
   switch (property_id)
     {
     case PROP_DATE:
-      g_value_set_boxed (value, priv->date);
+      g_value_set_boxed (value, self->date);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -614,15 +552,12 @@ gcal_search_view_get_property (GObject       *object,
 }
 
 static void
-gcal_search_view_finalize (GObject       *object)
+gcal_search_view_finalize (GObject *object)
 {
-  GcalSearchViewPrivate *priv;
-  priv = gcal_search_view_get_instance_private (GCAL_SEARCH_VIEW (object));
+  GcalSearchView *self = GCAL_SEARCH_VIEW (object);
 
-  if (priv->date != NULL)
-    g_free (priv->date);
-
-  g_hash_table_unref (priv->uuid_to_event);
+  g_clear_pointer (&self->date, g_free);
+  g_clear_pointer (&self->uuid_to_event, g_hash_table_unref);
 
   /* Chain up to parent's finalize() method. */
   G_OBJECT_CLASS (gcal_search_view_parent_class)->finalize (object);
@@ -634,42 +569,33 @@ gcal_search_view_component_added (ECalDataModelSubscriber *subscriber,
                                   ECalClient              *client,
                                   ECalComponent           *comp)
 {
-  GcalSearchViewPrivate *priv;
+  GcalSearchView *view;
 
   RowEventData *row_data;
-  GcalEventData *data;
-  ECalComponentId *id;
-  gchar *uuid;
+  GcalEvent *event;
+  gchar *uid;
 
-  priv = gcal_search_view_get_instance_private (GCAL_SEARCH_VIEW (subscriber));
+  view = GCAL_SEARCH_VIEW (subscriber);
 
-  /* event data */
-  data = g_new0 (GcalEventData, 1);
-  data->source = e_client_get_source (E_CLIENT (client));
-  data->event_component = e_cal_component_clone (comp);
-
-  /* build uuid */
-  id = e_cal_component_get_id (comp);
-
-  if (id->rid != NULL)
-    uuid = g_strdup_printf ("%s:%s:%s", e_source_get_uid (data->source), id->uid, id->rid);
-  else
-    uuid = g_strdup_printf ("%s:%s", e_source_get_uid (data->source), id->uid);
+  /* event */
+  event = gcal_event_new (e_client_get_source (E_CLIENT (client)), comp);
 
   /* insert row_data at the hash of events */
   row_data = g_new0 (RowEventData, 1);
-  row_data->event_data = data;
-  row_data->uuid = uuid;
-  row_data->row = make_row_for_event_data (GCAL_SEARCH_VIEW (subscriber), data);
+  row_data->event = event;
+  row_data->row = make_row_for_event (GCAL_SEARCH_VIEW (subscriber), event);
   g_signal_connect (row_data->row, "destroy", G_CALLBACK (gtk_widget_destroyed), &(row_data->row));
 
-  g_object_set_data (G_OBJECT (row_data->row), "event-data", row_data);
-  g_hash_table_insert (priv->uuid_to_event, uuid, row_data);
+  /* Add to the hash table */
+  uid = g_strdup (gcal_event_get_uid (event));
 
-  gtk_container_add (GTK_CONTAINER (priv->listbox), row_data->row);
+  g_object_set_data (G_OBJECT (row_data->row), "event-data", row_data);
+  g_hash_table_insert (view->uuid_to_event, uid, row_data);
+
+  gtk_container_add (GTK_CONTAINER (view->listbox), row_data->row);
 
   /* show 'no results' */
-  priv->num_results++;
+  view->num_results++;
 
   update_view (GCAL_SEARCH_VIEW (subscriber));
 }
@@ -688,12 +614,12 @@ gcal_search_view_component_removed (ECalDataModelSubscriber *subscriber,
                                     const gchar             *uid,
                                     const gchar             *rid)
 {
-  GcalSearchViewPrivate *priv;
+  GcalSearchView *view;
   ESource *source;
   RowEventData *row_data;
   gchar *uuid;
 
-  priv = gcal_search_view_get_instance_private (GCAL_SEARCH_VIEW (subscriber));
+  view = GCAL_SEARCH_VIEW (subscriber);
 
   /* if the widget has recurrency, it's UUID is different */
   source = e_client_get_source (E_CLIENT (client));
@@ -704,19 +630,19 @@ gcal_search_view_component_removed (ECalDataModelSubscriber *subscriber,
     uuid = g_strdup_printf ("%s:%s", e_source_get_uid (source), uid);
 
   // Lookup the RowEventData
-  row_data = g_hash_table_lookup (priv->uuid_to_event, uuid);
+  row_data = g_hash_table_lookup (view->uuid_to_event, uuid);
 
   /* Removing the given RowEventData entry will
    * call free_row_data, which removes the row
    * from the listbox and also free the uuid.
    */
   if (row_data)
-    g_hash_table_remove (priv->uuid_to_event, uuid);
+    g_hash_table_remove (view->uuid_to_event, uuid);
 
   g_free (uuid);
 
   /* show 'no results' */
-  priv->num_results--;
+  view->num_results--;
 
   update_view (GCAL_SEARCH_VIEW (subscriber));
 }
@@ -759,11 +685,8 @@ void
 gcal_search_view_connect (GcalSearchView *search_view,
                           GcalManager    *manager)
 {
-  GcalSearchViewPrivate *priv;
-
-  priv = gcal_search_view_get_instance_private (search_view);
-  if (manager != NULL && manager != priv->manager)
-    priv->manager = manager;
+  if (manager != NULL && manager != search_view->manager)
+    search_view->manager = manager;
 }
 
 /**
@@ -779,10 +702,7 @@ void
 gcal_search_view_set_time_format (GcalSearchView *view,
                                   gboolean        format_24h)
 {
-  GcalSearchViewPrivate *priv;
-
-  priv = gcal_search_view_get_instance_private (view);
-  priv->format_24h = format_24h;
+  view->format_24h = format_24h;
 }
 
 /**
@@ -800,18 +720,11 @@ gcal_search_view_search (GcalSearchView *view,
                          const gchar    *field,
                          const gchar    *query)
 {
-  GcalSearchViewPrivate *priv;
-
-  priv = gcal_search_view_get_instance_private (view);
-
-  if (priv->query != NULL)
-    g_free (priv->query);
-  if (priv->field != NULL)
-    g_free (priv->field);
-
-  priv->query = g_strdup (query);
-  priv->field = g_strdup (field);
+  g_clear_pointer (&view->query, g_free);
+  g_clear_pointer (&view->field, g_free);
 
+  view->query = g_strdup (query);
+  view->field = g_strdup (field);
 
   /* Only perform search on valid non-empty strings */
   if (query && g_utf8_strlen (query, -1) > 0)
@@ -819,22 +732,22 @@ gcal_search_view_search (GcalSearchView *view,
       gchar *search_query = g_strdup_printf ("(contains? \"%s\" \"%s\")", field != NULL? field : "summary",
                                              query != NULL? query : "");
 
-      if (!priv->subscribed)
+      if (!view->subscribed)
       {
-        gcal_manager_set_search_subscriber (priv->manager, E_CAL_DATA_MODEL_SUBSCRIBER (view), 0, 0);
-        priv->subscribed = TRUE;
+        gcal_manager_set_search_subscriber (view->manager, E_CAL_DATA_MODEL_SUBSCRIBER (view), 0, 0);
+        view->subscribed = TRUE;
       }
 
       /* update internal current time_t */
-      priv->current_utc_date = time (NULL);
+      view->current_utc_date = time (NULL);
 
-      gcal_manager_set_query (priv->manager, search_query);
+      gcal_manager_set_query (view->manager, search_query);
 
       g_free (search_query);
     }
   else
     {
-      g_hash_table_remove_all (priv->uuid_to_event);
+      g_hash_table_remove_all (view->uuid_to_event);
     }
 
   update_view (view);
