@@ -346,12 +346,9 @@ static void
 update_active_date (GcalWindow   *window,
                     icaltimetype *new_date)
 {
+  GDateTime *date_start, *date_end;
   time_t range_start, range_end;
-  icaltimetype date;
   icaltimetype *previous_date;
-  icaltimezone* default_zone;
-
-  default_zone = gcal_manager_get_system_timezone (window->manager);
 
   previous_date = window->active_date;
   window->active_date = new_date;
@@ -360,41 +357,31 @@ update_active_date (GcalWindow   *window,
   /* year_view */
   if (previous_date->year != new_date->year)
     {
-      date = *new_date;
-      date.day = 1;
-      date.month = 1;
-      date.hour = 0;
-      date.minute = 0;
-      date.second = 0;
-      date.is_date = 0;
-      range_start = icaltime_as_timet_with_zone (date, default_zone);
+      date_start = g_date_time_new_local (new_date->year, 1, 1, 0, 0, 0);
+      range_start = g_date_time_to_unix (date_start);
 
-      date.day = 31;
-      date.month = 12;
-      date.hour = 23;
-      date.minute = 59;
-      range_end = icaltime_as_timet_with_zone (date, default_zone);
+      date_end = g_date_time_add_years (date_start, 1);
+      range_end = g_date_time_to_unix (date_end);
 
       gcal_manager_set_subscriber (window->manager, E_CAL_DATA_MODEL_SUBSCRIBER (window->year_view), range_start, range_end);
+
+      g_clear_pointer (&date_start, g_date_time_unref);
+      g_clear_pointer (&date_end, g_date_time_unref);
     }
 
   /* month_view */
   if (previous_date->month != new_date->month || previous_date->year != new_date->year)
     {
-      date = *new_date;
-      date.day = 1;
-      date.hour = 0;
-      date.minute = 0;
-      date.second = 0;
-      date.is_date = 0;
-      range_start = icaltime_as_timet_with_zone (date, default_zone);
+      date_start = g_date_time_new_local (new_date->year, new_date->month, 1, 0, 0, 0);
+      range_start = g_date_time_to_unix (date_start);
 
-      date.day = time_days_in_month (new_date->year, new_date->month - 1);
-      date.hour = 23;
-      date.minute = 59;
-      range_end = icaltime_as_timet_with_zone (date, default_zone);
+      date_end = g_date_time_add_months (date_start, 1);
+      range_end = g_date_time_to_unix (date_end);
 
       gcal_manager_set_subscriber (window->manager, E_CAL_DATA_MODEL_SUBSCRIBER (window->month_view), range_start, range_end);
+
+      g_clear_pointer (&date_start, g_date_time_unref);
+      g_clear_pointer (&date_end, g_date_time_unref);
     }
 
   g_free (previous_date);
@@ -680,8 +667,8 @@ show_new_event_widget (GcalView   *view,
 
   if (window->event_creation_data != NULL)
     {
-      g_date_time_unref (window->event_creation_data->start_date);
-      g_date_time_unref (window->event_creation_data->end_date);
+      g_clear_pointer (&window->event_creation_data->start_date, g_date_time_unref);
+      g_clear_pointer (&window->event_creation_data->end_date, g_date_time_unref);
       g_free (window->event_creation_data);
     }
 
@@ -871,7 +858,7 @@ show_source_dialog (GtkButton *button,
 
   gtk_widget_hide (window->calendar_popover);
 
-  gtk_dialog_run (GTK_DIALOG (window->source_dialog));
+  gtk_widget_show (window->source_dialog);
 }
 
 static void
@@ -893,7 +880,7 @@ source_row_activated (GtkListBox    *listbox,
 
   gtk_widget_hide (window->calendar_popover);
 
-  gtk_dialog_run (GTK_DIALOG (window->source_dialog));
+  gtk_widget_show (window->source_dialog);
 }
 
 static void
@@ -998,7 +985,7 @@ edit_event (GcalQuickAddPopover *popover,
   gcal_edit_dialog_set_event_is_new (GCAL_EDIT_DIALOG (window->edit_dialog), TRUE);
   gcal_edit_dialog_set_event (GCAL_EDIT_DIALOG (window->edit_dialog), event);
 
-  gtk_dialog_run (GTK_DIALOG (window->edit_dialog));
+  gtk_widget_show (window->edit_dialog);
 }
 
 static void
@@ -1017,7 +1004,7 @@ create_event_detailed_cb (GcalView *view,
   gcal_edit_dialog_set_event_is_new (GCAL_EDIT_DIALOG (window->edit_dialog), TRUE);
   gcal_edit_dialog_set_event (GCAL_EDIT_DIALOG (window->edit_dialog), event);
 
-  gtk_dialog_run (GTK_DIALOG (window->edit_dialog));
+  gtk_widget_show (window->edit_dialog);
 
   g_clear_object (&comp);
 }
@@ -1034,7 +1021,7 @@ event_activated (GcalView        *view,
   gcal_edit_dialog_set_event_is_new (GCAL_EDIT_DIALOG (window->edit_dialog), FALSE);
   gcal_edit_dialog_set_event (GCAL_EDIT_DIALOG (window->edit_dialog), event);
 
-  gtk_dialog_run (GTK_DIALOG (window->edit_dialog));
+  gtk_widget_show (window->edit_dialog);
 }
 
 static void
@@ -1403,6 +1390,13 @@ gcal_window_finalize (GObject *object)
       window->save_geometry_timeout_id = 0;
     }
 
+  /* If we have a queued event to delete, remove it now */
+  if (window->event_to_delete)
+    {
+      gcal_manager_remove_event (window->manager, window->event_to_delete);
+      g_clear_object (&window->event_to_delete);
+    }
+
   g_clear_object (&window->manager);
   g_clear_object (&window->views_switcher);
 
