@@ -29,6 +29,7 @@
 #define NAVIGATOR_CELL_HEIGHT 210
 #define SIDEBAR_PREFERRED_WIDTH 200
 #define VISUAL_CLUES_SIDE 3.0
+#define WEEK_NUMBER_PADDING 3.0
 
 typedef struct
 {
@@ -150,7 +151,7 @@ static void
 update_date (GcalYearView *year_view,
              icaltimetype *new_date)
 {
-  if (year_view->date != NULL && year_view->date->year != new_date->year && year_view->start_selected_date->day != 0)
+  if (year_view->date != NULL && icaltime_compare_date (year_view->date, new_date) && year_view->start_selected_date->day != 0)
     reset_sidebar (year_view);
 
   g_clear_pointer (&year_view->date, g_free);
@@ -448,6 +449,7 @@ update_sidebar_headers (GtkListBoxRow *row,
 
   row_event = gcal_event_widget_get_event (GCAL_EVENT_WIDGET (row_child));
   row_date = gcal_event_get_date_start (row_event);
+  row_date = g_date_time_to_local (row_date);
   row_shift = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (row_child), "shift"));
 
   if (before != NULL)
@@ -455,6 +457,7 @@ update_sidebar_headers (GtkListBoxRow *row,
       before_child = gtk_bin_get_child (GTK_BIN (before));
       before_event = gcal_event_widget_get_event (GCAL_EVENT_WIDGET (before_child));
       before_date = gcal_event_get_date_start (before_event);
+      before_date = g_date_time_to_local (before_date);
       before_shift = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (before_child), "shift"));
     }
 
@@ -511,6 +514,9 @@ update_sidebar_headers (GtkListBoxRow *row,
   if (row_header != NULL)
     gtk_widget_show_all (row_header);
   gtk_list_box_row_set_header (row, row_header);
+
+  g_clear_pointer (&before_date, g_date_time_unref);
+  g_clear_pointer (&row_date, g_date_time_unref);
 }
 
 static gint
@@ -838,11 +844,16 @@ draw_month_grid (GcalYearView *year_view,
 
       pango_layout_set_text (layout, nr_week, -1);
       pango_layout_get_pixel_size (layout, &layout_width, &layout_height);
-      box_padding_top = (box_side - layout_height) / 2 > 0 ? (box_side - layout_height) / 2 : 0;
-      box_padding_start = ((box_side / 2) - layout_width) / 2 > 0 ? ((box_side / 2) - layout_width) / 2 : 0;
+      box_padding_top = (box_side - layout_height) / 2.0 > 0 ? (box_side - layout_height) / 2.0 : 0;
+      box_padding_start = (box_side - layout_width) / 2.0 > 0 ? (box_side - layout_width) / 2.0 : 0;
+
+      gtk_render_background (context, cr,
+                             box_side * (- 0.5) + x + sw * WEEK_NUMBER_PADDING + year_view->k * (8 * box_side + WEEK_NUMBER_PADDING * 2),
+                             box_side * (i + 1) + y + WEEK_NUMBER_PADDING,
+                             box_side - WEEK_NUMBER_PADDING * 2, box_side - WEEK_NUMBER_PADDING * 2);
 
       gtk_render_layout (context, cr,
-                         x + sw * box_padding_start + year_view->k * (8 * box_side - layout_width),
+                         box_side * (- 0.5) + x + sw * box_padding_start + year_view->k * (9 * box_side - layout_width),
                          box_side * (i + 1) + y + box_padding_top,
                          layout);
 
